// coding: utf-8
/*! Created by: Udo Schmal | https://www.gocher.me/ */
(function () {
  'use strict';

  function fullscreen (btn) {
    if (btn) {
      var language = window.navigator.userLanguage || window.navigator.language;
      if (document.fullScreenEnabled || document.mozFullScreenEnabled || document.webkitFullScreenEnabled || document.msFullscreenEnabled) {
        const
          lang = (language ? language.substr(0, 2) : 'en'),
          txtRequestFullscreen = (lang == 'de' ? 'Vollbild-Modus' : 'toggle to fullscreen mode'),
          txtCancelFullscreen = (lang == 'de' ? 'Vollbild-Modus beenden' : 'cancel fullscreen mode [Esc]');
        function setStatus () {
          if (!document.fullscreenElement && !document.mozFullScreenElement && !document.webkitFullscreenElement && !document.msFullscreenElement) {
            btn.classList.remove('active');
            btn.setAttribute('title', txtRequestFullscreen);
          } else {
            btn.classList.add('active');
            btn.setAttribute('title', txtCancelFullscreen);
          }
        }
        function toggle () {
          if (!document.fullscreenElement && !document.mozFullScreenElement && !document.webkitFullscreenElement && !document.msFullscreenElement) {
            var body = document.body;
            if (body.requestFullscreen) {
              body.requestFullscreen();
            } else if (body.msRequestFullscreen) {
              body.msRequestFullscreen();
            } else if (body.mozRequestFullScreen) {
              body.mozRequestFullScreen();
            } else if (body.webkitRequestFullscreen) {
              body.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
            }
          } else {
            if (document.exitFullscreen) {
              document.exitFullscreen();
            } else if (document.msExitFullscreen) {
              document.msExitFullscreen();
            } else if (document.mozCancelFullScreen) {
              document.mozCancelFullScreen();
            } else if (document.webkitExitFullscreen) {
              document.webkitExitFullscreen();
            } else if (document.webkitCancelFullScreen) {
              document.webkitCancelFullScreen();
            }
          }
        }
        btn.addEventListener('click', toggle);
        if (document.fullScreenEnabled) {
          document.addEventListener('fullscreenchange', setStatus);
        } else if (document.mozFullScreenEnabled) {
          document.addEventListener('mozfullscreenchange', setStatus);
        } else if (document.webkitFullScreenEnabled) {
          document.addEventListener('webkitfullscreenchange', setStatus);
        } else if (document.msFullscreenEnabled) {
          document.addEventListener('MSFullscreenChange', setStatus);
        }
        setStatus();
      } else {
        btn.style.display = 'none';
      }
    }
  }

  function Gallery() {
    this.activeLink = null;
    this.childs = null;
    this.slideshow = null;
    this.keyListener = false;
    this.hd = localStorage.getItem('gallery_hd') == 'true';
    this.pixelratio = (typeof(window.devicePixelRatio)!="undefined") ? window.devicePixelRatio : "1";
    this.disp = Math.max(screen.width, screen.height) * this.pixelratio;
    this.touchsupported = ('ontouchstart' in window) || (navigator.MaxTouchPoints) || (navigator.msMaxTouchPoints);
    if (this.hd) {
      if (this.touchsupported) {
        this.view = this.disp;
      } else {
        this.view = '' + (screen.width * this.pixelratio) + 'x' + (screen.height * this.pixelratio);
      }
    } else {
      this.view = this.disp<1024 ? this.disp : 1024;
    }
    this.Overlay = null;
    this.Box = null;
    this.Caption = null;
    this.Image = null;
    this.Temp = null;
    this.PreviousBtn = null;
    this.NextBtn = null;
    this.SlideBtn = null;
    this.DownloadBtn = null;
    this.HDBtn = null;
    this.href = null;

  // handle touch event
    this.fingerCount = 0;
    this.startX = 0;
    this.startY = 0;
    this.curX = 0;
    this.curY = 0;
    this.deltaX = 0;
    this.deltaY = 0;
    this.horzDiff = 0;
    this.vertDiff = 0;
    this.minLength = 72; // the shortest distance the user may swipe
    this.swipeLength = 0;
    this.swipeAngle = null;
    this.swipeDirection = null;
    this.language = window.navigator.userLanguage || window.navigator.language;
    if (!this.language) {
      this.language = 'en';
    }
    this.txtprevious = unescape((this.language=='de' ? 'vorheriges' : 'previous') + ' [%u2190]');
    this.txtnext = unescape((this.language=='de' ? 'n%E4chstes' : 'next') + ' [%u2192]');
    this.txtslideshow = 'Diashow [start]';
    this.txtslidestop = 'Diashow [stop]';
    this.txtdownload = unescape(this.language=='de' ? 'Hochaufl%F6sendes Bild herunterladen' : 'download full size image');
    this.txtclose = unescape((this.language=='de' ? 'schlie%DFen' : 'close') + ' [Esc]');
    this.txthd = 'HD';
    this.txtnohd = 'no HD';
  }
  Gallery.prototype = {
    toggleHD: function() {
      this.hd = !this.hd;
      localStorage.setItem('gallery_hd', ''+this.hd);
      if (this.hd) {
        if (this.touchsupported) {
          this.view = this.disp;
        } else {
          this.view = '' + (screen.width * this.pixelratio) + 'x' + (screen.height * this.pixelratio);
        }
        this.Image.src = this.href.substring(0, this.href.lastIndexOf('.')) + '~' + this.view + this.href.substring(this.href.lastIndexOf('.'));
      } else {
        this.view = this.disp<1024 ? this.disp : 1024;
      }
      if (this.hd) {
        this.HDBtn.classList.add('active');
        this.HDBtn.setAttribute('title', this.txtnohd);
      } else {
        this.HDBtn.classList.remove('active');
        this.HDBtn.setAttribute('title', this.txthd);
      }
    },
    showFirst: function () {
      if (this.slideshow) {
        window.clearInterval(this.slideshow);
      }
      this.show(this.childs[0]);
    },
    showPrevious: function () {
      if (this.slideshow) {
        window.clearInterval(this.slideshow);
      }
      for (var i=this.childs.length-1; i>=0; i--) {
        if ((i>0) && (this.childs[i]==this.activeLink) && (this.childs[i-1])) {
          this.show(this.childs[i-1]);
          break;
        }
      }
    },
    showNext: function () {
      for (var i=0, len=this.childs.length; i<len; i++) {
        if ((i<len) && (this.childs[i]==this.activeLink) && (this.childs[i+1])) {
          this.show(this.childs[i+1]);
          return;
        }
      }
      this.show(this.childs[0]);
    },
    showLast: function () {
      if (this.slideshow) {
        window.clearInterval(this.slideshow);
      }
      this.show(this.childs[this.childs.length-1]);
    },
    slideShow: function () {
      if (this.SlideBtn.classList.contains('active')) {
        if (this.slideshow) {
          window.clearInterval(this.slideshow);
        }
        this.slideshow = null;
        // hide slideBtn if only singel image
        this.SlideBtn.style.display = this.childs.length > 1 ? 'block' : 'none';
        this.SlideBtn.classList.remove('active');
        this.SlideBtn.setAttribute('title', this.txtslideshow);
      } else {
        var self = this;
        // hide slideBtn if only singel image
        this.SlideBtn.classList.add('active');
        this.SlideBtn.setAttribute('title', this.txtslidestop);
        this.slideshow = window.setInterval(function () {
            self.showNext();
          }, 3000);
      }
    },
    getKey: function (event) {
      var handled = false;
      event = event || window.event;
      var keycode = event.keyCode;
      var key = String.fromCharCode(keycode).toLowerCase();
      if ((keycode != 19) && (this.slideshow)) {
        window.clearInterval(this.slideshow);
      }
      //alert(keycode);
      switch (keycode) {
        case 38:
          this.showFirst();
          handled = true;
          break;
        case 37:
          this.showPrevious();
          handled = true;
          break;
        case 39:
          this.showNext();
          handled = true;
          break;
        case 40:
          this.showLast();
          handled = true;
          break;
        case 27:
          this.hide();
          handled = true;
          break;
        case 19:
          if (this.slideshow) {
            window.clearInterval(this.slideshow);
            this.slideshow = null;
          } else {
            this.slideShow();
          }
          handled = true;
          break;
      }
      if (handled) {
        //IE9 & Other Browsers
        if (event.stopPropagation) {
          event.stopPropagation();
        } else { //IE8 and Lower
          event.cancelBubble = true;
        }
      }
      return handled==false;
    },
    show: function (objLink) {
      var self = this;
      this.activeLink = objLink;
      // show overlay image viewer
      if (this.Overlay.style.display != 'block') {
        this.Overlay.style.display = 'block';
        // enable keydown listener
        if (this.keyListener == false) {
          document.addEventListener('keydown', function (event) {
            self.getKey(event);
            return false;
          });
          this.keyListener = true;
        }
      }
      // get images from gallery
      if (this.activeLink.className == 'hires') {
        var place = this.activeLink.parentNode;
        if (place.tagName.toLowerCase() == 'li') {
          place = place.parentNode;
        }
      } else {
        var place = this.activeLink.parentNode.parentNode;
      }
      if (place) {
        this.childs = place.getElementsByTagName("a");
      }
      // hide prevBtn if first and nextBtn if last
      this.PreviousBtn.style.display = (this.childs[0] == this.activeLink) ? 'none' : 'block';
      this.NextBtn.style.display = (this.childs[this.childs.length-1] == this.activeLink) ? 'none' : 'block';
      // hide slideBtn if only singel image
      if (!this.slideshow) {
        this.SlideBtn.style.display = this.childs.length > 1 ? 'block' : 'none';
        this.SlideBtn.classList.remove('active');
        this.SlideBtn.setAttribute('title', this.txtslideshow);
      }
      // get image with screen size
      this.href = objLink.href;
      this.Image.src = objLink.href.substring(0, objLink.href.lastIndexOf('.')) + '~' + this.view + objLink.href.substring(objLink.href.lastIndexOf('.'));
      // copy values from gallery
      if (objLink.getAttribute('title')) {
        this.Caption.style.display = 'block';
        this.Image.setAttribute('alt', objLink.getAttribute('title'));
        this.Caption.innerHTML = '<p>' + objLink.getAttribute('title') + '</p>';
      } else {
        this.Caption.style.display = 'none';
      }
      this.DownloadBtn.href = objLink.href;
    },
    hide: function () {
      var self = this;
      // hide overlay image viewer
      this.Overlay.style.display = 'none';
      if (this.slideshow) {
        window.clearInterval(this.slideshow);
      }
      this.slideshow = null;
      // disable keydown listener
  //    document.removeEventListener('keydown', function (event) { self.getKey(event); return false; });
    },
    opacity: function (el) {
      if (el.xOpacity > 0.99) {
        el.xOpacity = 0.99;
      }
      el.style.opacity = el.xOpacity;
      el.style.MozOpacity = el.xOpacity;
      el.style.filter = "alpha(opacity=" + (el.xOpacity * 100) + ")";
    },
    tempfade: function (px) {
      if (px > 1) {
        var tempopacity = px / (this.Image.offsetWidth / 2);
        var href = this.Temp.getAttribute('data-src');
        this.Temp.src = href.substring(0, href.lastIndexOf('.')) + '~' + this.view + href.substring(href.lastIndexOf('.'));
        this.Temp.style.display = 'block';
        this.Temp.xOpacity = tempopacity;
        this.opacity(this.Temp);
        this.Image.xOpacity = 1 - tempopacity;
        this.opacity(this.Image);
      } else {
        this.Image.xOpacity = 1.0;
        this.opacity(this.Image);
        this.Temp.style.display = 'none';
        this.Temp.xOpacity = 0.0;
        this.opacity(this.Temp);
      }
    },
    determineSwipeDirection: function () {
      var X = this.startX-this.curX;
      var Y = this.curY-this.startY;
      this.swipeLength = Math.round(Math.sqrt(Math.pow(X,2)+Math.pow(Y,2))); //the distance - rounded - in pixels
      var r = Math.atan2(Y, X); //angle in radians (Cartesian system)
      this.swipeAngle = Math.round(r*180/Math.PI); //angle in degrees
      if (this.swipeAngle < 0) {
        this.swipeAngle = 360 - Math.abs(this.swipeAngle);
      }
      if ((this.swipeAngle <= 45) && (this.swipeAngle >= 0)) {
        this.swipeDirection = 'left';
      } else if ((this.swipeAngle <= 360) && (this.swipeAngle >= 315)) {
        this.swipeDirection = 'left';
      } else if ((this.swipeAngle >= 135) && (this.swipeAngle <= 225)) {
        this.swipeDirection = 'right';
      } else if ((this.swipeAngle > 45) && (this.swipeAngle < 135)) {
        this.swipeDirection = 'down';
      } else {
        this.swipeDirection = 'up';
      }
    },
    touchCancel: function (event) {
      this.tempfade(0);
      // reset the variables back to default values
      this.fingerCount = 0;
      this.startX = 0;
      this.startY = 0;
      this.curX = 0;
      this.curY = 0;
      this.deltaX = 0;
      this.deltaY = 0;
      this.horzDiff = 0;
      this.vertDiff = 0;
      this.swipeLength = 0;
      this.swipeAngle = null;
      this.swipeDirection = null;
    },
    touchStart: function (event) {
      // disable the standard ability to select the touched object
      //event.preventDefault(); // no onclick supported
  //    clearTimeout(this.slideref);
      // get the total number of fingers touching the screen
      this.fingerCount = event.touches.length;
      // since we're looking for a swipe (single finger) and not a gesture (multiple fingers),
      // check that only one finger was used
      if (this.fingerCount === 1) {
        // get the coordinates of the touch
        this.startX = event.touches[0].pageX;
        this.startY = event.touches[0].pageY;
      } else {
        // more than one finger touched so cancel
        this.touchCancel(event);
      }
    },
    touchMove: function (event) {
      //event.preventDefault();
      if (event.touches.length === 1) {
        this.curX = event.touches[0].pageX;
        this.curY = event.touches[0].pageY;
        this.determineSwipeDirection();
        var href;
        if (this.swipeDirection==='left') {
          event.preventDefault();
          href = this.childs[0];
          for (var i=0, len=this.childs.length; i<len; i++) {
            if ((i<len) && (this.childs[i]==this.activeLink) && (this.childs[i+1])) {
              href = this.childs[i+1].href;
              break;
            }
          }
          this.Temp.setAttribute('data-src', href);
          this.tempfade(this.swipeLength);
        } else if (this.swipeDirection==='right') {
          event.preventDefault();
          href = this.childs[this.childs.length-1];
          for (var i=this.childs.length-1; i>=0; i--) {
            if ((i>0) && (this.childs[i]==this.activeLink) && (this.childs[i-1])) {
              href = this.childs[i-1];
              break;
            }
          }
          this.Temp.setAttribute('data-src', href);
          this.tempfade(this.swipeLength);
        }
      } else {
        this.touchCancel(event);
      }
    },
    touchEnd: function (event) {
      //event.preventDefault();
      // check to see if more than one finger was used and that there is an ending coordinate
      if (this.fingerCount === 1 && this.curX !== 0) {
        // use the Distance Formula to determine the length of the swipe
        this.swipeLength = Math.round(Math.sqrt(Math.pow(this.curX - this.startX,2) + Math.pow(this.curY - this.startY,2)));
        // if the user swiped more than the minimum length, perform the appropriate action
        if (this.swipeLength >= this.minLength ) {
          this.determineSwipeDirection();
          this.processingRoutine();
          this.touchCancel(event); // reset the variables
        } else {
          this.touchCancel(event);
        }
      } else {
        this.touchCancel(event);
      }
    },
    processingRoutine: function () {
      if (this.swipeDirection === 'left') {
        this.showNext();
      } else if (this.swipeDirection === 'right') {
        this.showPrevious();
      }
    },
    init: function () {
      var self = this;

      // create overlay div
      this.Overlay = document.getElementById("overlay");
      if (!this.Overlay) {
        this.Overlay = document.createElement("div");
        this.Overlay.setAttribute('id','overlay');
        this.Overlay.style.display = 'none';
        document.body.appendChild(this.Overlay);
      }
      // create image container
      this.Box = document.createElement('div');
      this.Box.setAttribute('id','imagebox');
      this.Overlay.appendChild(this.Box);
      if (this.touchsupported) {
        this.Box.addEventListener('touchstart', function (event) {self.touchStart(event);});
        this.Box.addEventListener('touchend', function (event) {self.touchEnd(event);});
        this.Box.addEventListener('touchmove', function (event) {self.touchMove(event);});
        this.Box.addEventListener('touchcancel', function (event) {self.touchCancel(event);});
      }

      // create image
      this.Image = document.createElement('img');
      this.Image.setAttribute('id','image');
      this.Image.src = "data:,";
      this.Box.appendChild(this.Image);
      this.Temp = document.createElement('img');
      this.Temp.setAttribute('id','temp');
      this.Temp.style.display = 'none';
      this.Temp.src = "data:,";
      this.Box.appendChild(this.Temp);

      // create buttons
      this.PreviousBtn = document.createElement("span");
      this.PreviousBtn.className = 'previous';
      this.PreviousBtn.setAttribute('title', this.txtprevious);
      this.PreviousBtn.addEventListener('click', function (event) {
        self.showPrevious();
      });
      this.Overlay.appendChild(this.PreviousBtn);

      this.NextBtn = document.createElement("span");
      this.NextBtn.className = 'next';
      this.NextBtn.setAttribute('title', this.txtnext);
      this.NextBtn.addEventListener('click', function (event) {
        self.showNext();
      });
      this.Overlay.appendChild(this.NextBtn);

      if (!this.touchsupported) {
        var fullscreenBtn = document.createElement("span");
        fullscreenBtn.className = 'fullscreen';
        fullscreen(fullscreenBtn);
        this.Overlay.appendChild(fullscreenBtn);
      }
      var closeBtn = document.createElement("span");
      closeBtn.className = 'close';
      closeBtn.setAttribute('title', this.txtclose);
      closeBtn.addEventListener('click', function (event) {
        self.hide();
      });
      this.Overlay.appendChild(closeBtn);

      this.SlideBtn = document.createElement("span");
      this.SlideBtn.className = 'slideshow';
      this.SlideBtn.setAttribute('title', this.txtslideshow);
      this.SlideBtn.addEventListener('click', function (event) {
        self.slideShow();
      });
      this.Overlay.appendChild(this.SlideBtn);

      this.DownloadBtn = document.createElement("span");
      this.DownloadBtn.className = 'download';
      this.DownloadBtn.setAttribute('title', this.txtdownload);
      this.DownloadBtn.addEventListener('click', function (event) {
        window.open(this.href);
      });
      this.Overlay.appendChild(this.DownloadBtn);

      if (this.disp>1024) {
        this.HDBtn = document.createElement("span");
        this.HDBtn.className = 'hd';
        this.HDBtn.setAttribute('title', this.txthd);
        this.HDBtn.addEventListener('click', function (event) {
          self.toggleHD();
        });
        this.Overlay.appendChild(this.HDBtn);
        if (this.hd) {
          this.HDBtn.classList.add('active');
          this.HDBtn.setAttribute('title', this.txtnohd);
        } else {
          this.HDBtn.classList.remove('active');
          this.HDBtn.setAttribute('title', this.txthd);
        }
      }
      // create caption
      this.Caption = document.createElement("div");
      this.Caption.setAttribute('id', 'caption');
      this.Overlay.appendChild(this.Caption);
    }
  };

  // init galleryViewer
  var galleryViewer = null;
  let hires = document.querySelectorAll('a.hires');
  let galleryImages = document.querySelectorAll('ul.gallery a');
  if ((hires.length > 0) || (galleryImages.length > 0)) {
    galleryViewer = new Gallery();
    galleryViewer.init();
    for (let i=0; i<hires.length; i++) {
      hires[i].addEventListener('click', function (event) {
        event.preventDefault();
        event.stopPropagation();
        galleryViewer.show(this);
        this.blur();
      });
    }
    for (let i=0; i<galleryImages.length; i++) {
      galleryImages[i].addEventListener('click', function (event) {
        event.preventDefault();
        event.stopPropagation();
        galleryViewer.show(this);
        this.blur();
      });
    }
  }
})();
